@@ -1,14 +1,15 @@
-# SPDX-License-Identifier: LGPL-2.1+
+# SPDX-License-Identifier: LGPL-2.1-or-later
 
 import os
-from collections.abc import Iterable
+from collections.abc import Iterable, Sequence
 from pathlib import Path
 from typing import Optional
 
 from mkosi.log import log_step
 from mkosi.run import run
 from mkosi.sandbox import Mount, SandboxProtocol, finalize_passwd_mounts, nosandbox
-from mkosi.util import umask
+from mkosi.types import PathString
+from mkosi.util import chdir, umask
 
 
 def tar_exclude_apivfs_tmp() -> list[str]:
@@ -50,11 +51,16 @@ def make_tar(src: Path, dst: Path, *, sandbox: SandboxProtocol = nosandbox) -> N
         )
 
 
+def can_extract_tar(src: Path) -> bool:
+    return ".tar" in src.suffixes[-2:]
+
+
 def extract_tar(
     src: Path,
     dst: Path,
     *,
     log: bool = True,
+    options: Sequence[PathString] = (),
     sandbox: SandboxProtocol = nosandbox,
 ) -> None:
     if log:
@@ -63,31 +69,30 @@ def extract_tar(
     with umask(~0o755):
         dst.mkdir(exist_ok=True)
 
-    with src.open("rb") as f:
-        run(
-            [
-                "tar",
-                "--extract",
-                "--file", "-",
-                "--directory", dst,
-                "--keep-directory-symlink",
-                "--no-overwrite-dir",
-                "--same-permissions",
-                "--same-owner" if (dst / "etc/passwd").exists() else "--numeric-owner",
-                "--same-order",
-                "--acls",
-                "--selinux",
-                "--xattrs",
-                "--force-local",
-                *tar_exclude_apivfs_tmp(),
-            ],
-            stdin=f,
-            sandbox=sandbox(
-                binary="tar",
-                # Make sure tar uses user/group information from the root directory instead of the host.
-                mounts=[Mount(src, src, ro=True), Mount(dst, dst), *finalize_passwd_mounts(dst)]
-            ),
-        )
+    run(
+        [
+            "tar",
+            "--extract",
+            "--file", src,
+            "--directory", dst,
+            "--keep-directory-symlink",
+            "--no-overwrite-dir",
+            "--same-permissions",
+            "--same-owner" if (dst / "etc/passwd").exists() else "--numeric-owner",
+            "--same-order",
+            "--acls",
+            "--selinux",
+            "--xattrs",
+            "--force-local",
+            *tar_exclude_apivfs_tmp(),
+            *options,
+        ],
+        sandbox=sandbox(
+            binary="tar",
+            # Make sure tar uses user/group information from the root directory instead of the host.
+            mounts=[Mount(src, src, ro=True), Mount(dst, dst), *finalize_passwd_mounts(dst)]
+        ),
+    )
 
 
 def make_cpio(
@@ -98,8 +103,10 @@ def make_cpio(
     sandbox: SandboxProtocol = nosandbox,
 ) -> None:
     if not files:
-        files = src.rglob("*")
-    files = sorted(files)
+        with chdir(src):
+            files = sorted(Path(".").rglob("*"))
+    else:
+        files = sorted(files)
 
     log_step(f"Creating cpio archive {dst}…")
 
@@ -114,7 +121,7 @@ def make_cpio(
                 "--quiet",
                 "--directory", src,
             ],
-            input="\0".join(os.fspath(f.relative_to(src)) for f in files),
+            input="\0".join(os.fspath(f) for f in files),
             stdout=f,
             sandbox=sandbox(binary="cpio", mounts=[Mount(src, src, ro=True), *finalize_passwd_mounts(src)]),
         )
