@@ -1,4 +1,4 @@
-# SPDX-License-Identifier: LGPL-2.1+
+# SPDX-License-Identifier: LGPL-2.1-or-later
 
 from collections.abc import Sequence
 from contextlib import AbstractContextManager
@@ -23,12 +23,15 @@ class Context:
         workspace: Path,
         resources: Path,
         package_cache_dir: Optional[Path] = None,
+        package_dir: Optional[Path] = None,
     ) -> None:
         self.args = args
         self.config = config
         self.workspace = workspace
         self.resources = resources
         self.package_cache_dir = package_cache_dir or (self.root / "var")
+        self.package_dir = package_dir or (self.workspace / "packages")
+        self.package_dir.mkdir(exist_ok=True)
 
         with umask(~0o755):
             # Using a btrfs subvolume as the upperdir in an overlayfs results in EXDEV so make sure we create
@@ -44,7 +47,7 @@ class Context:
 
         self.staging.mkdir()
         self.pkgmngr.mkdir()
-        self.packages.mkdir()
+        self.repository.mkdir()
         self.artifacts.mkdir()
         self.install_dir.mkdir()
 
@@ -61,8 +64,8 @@ class Context:
         return self.workspace / "pkgmngr"
 
     @property
-    def packages(self) -> Path:
-        return self.workspace / "packages"
+    def repository(self) -> Path:
+        return self.workspace / "repository"
 
     @property
     def artifacts(self) -> Path:
@@ -78,6 +81,7 @@ class Context:
         binary: Optional[PathString],
         network: bool = False,
         devices: bool = False,
+        vartmp: bool = False,
         scripts: Optional[Path] = None,
         mounts: Sequence[Mount] = (),
         options: Sequence[PathString] = (),
@@ -95,6 +99,7 @@ class Context:
             binary=binary,
             network=network,
             devices=devices,
+            vartmp=vartmp,
             scripts=scripts,
             mounts=[
                 # This mount is writable so bubblewrap can create extra directories or symlinks inside of it as needed.
@@ -113,6 +118,3 @@ class Context:
             ],
             extra=extra,
         )
-
-    def want_local_repo(self) -> bool:
-        return any(self.packages.iterdir())
