@@ -1,4 +1,4 @@
-# SPDX-License-Identifier: LGPL-2.1+
+# SPDX-License-Identifier: LGPL-2.1-or-later
 
 import enum
 import importlib
@@ -71,10 +71,6 @@ class DistributionInstaller:
     def grub_prefix(cls) -> str:
         return "grub"
 
-    @classmethod
-    def createrepo(cls, context: "Context") -> None:
-        raise NotImplementedError
-
 
 class Distribution(StrEnum):
     # Please consult docs/distribution-policy.md and contact one
@@ -135,14 +131,14 @@ class Distribution(StrEnum):
     def default_release(self) -> str:
         return self.installer().default_release()
 
-    def default_tools_tree_distribution(self) -> Optional["Distribution"]:
-        return self.installer().default_tools_tree_distribution()
+    def default_tools_tree_distribution(self) -> "Distribution":
+        return self.installer().default_tools_tree_distribution() or self
 
     def grub_prefix(self) -> str:
         return self.installer().grub_prefix()
 
     def createrepo(self, context: "Context") -> None:
-        return self.installer().createrepo(context)
+        return self.installer().package_manager(context.config).createrepo(context)
 
     def installer(self) -> type[DistributionInstaller]:
         modname = str(self).replace('-', '_')
