@@ -1,12 +1,12 @@
-# SPDX-License-Identifier: LGPL-2.1+
+# SPDX-License-Identifier: LGPL-2.1-or-later
 
 from collections.abc import Iterable, Sequence
 
 from mkosi.config import Architecture, Config
 from mkosi.context import Context
-from mkosi.distributions import Distribution, DistributionInstaller, PackageType
+from mkosi.distributions import DistributionInstaller, PackageType
 from mkosi.installer import PackageManager
-from mkosi.installer.pacman import Pacman
+from mkosi.installer.pacman import Pacman, PacmanRepository
 from mkosi.log import die
 from mkosi.util import listify, sort_packages
 
@@ -28,18 +28,10 @@ class Installer(DistributionInstaller):
     def default_release(cls) -> str:
         return "rolling"
 
-    @classmethod
-    def default_tools_tree_distribution(cls) -> Distribution:
-        return Distribution.arch
-
     @classmethod
     def package_manager(cls, config: "Config") -> type[PackageManager]:
         return Pacman
 
-    @classmethod
-    def createrepo(cls, context: Context) -> None:
-        Pacman.createrepo(context)
-
     @classmethod
     def setup(cls, context: Context) -> None:
         Pacman.setup(context, cls.repositories(context))
@@ -63,9 +55,9 @@ class Installer(DistributionInstaller):
 
     @classmethod
     @listify
-    def repositories(cls, context: Context) -> Iterable[Pacman.Repository]:
+    def repositories(cls, context: Context) -> Iterable[PacmanRepository]:
         if context.config.local_mirror:
-            yield Pacman.Repository("core", context.config.local_mirror)
+            yield PacmanRepository("core", context.config.local_mirror)
         else:
             if context.config.architecture.is_arm_variant():
                 url = f"{context.config.mirror or 'http://mirror.archlinuxarm.org'}/$arch/$repo"
@@ -81,11 +73,16 @@ class Installer(DistributionInstaller):
                     "extra-testing-debug",
                     "core-debug",
                     "extra-debug",
+                    "multilib-testing",
+                    "multilib",
                 ) if repo in context.config.repositories
             ] + ["core", "extra"]
 
+            if context.config.architecture.is_arm_variant():
+                repos += ["alarm"]
+
             for repo in repos:
-                yield Pacman.Repository(repo, url)
+                yield PacmanRepository(repo, url)
 
     @classmethod
     def architecture(cls, arch: Architecture) -> str:
