@@ -1,4 +1,4 @@
-# SPDX-License-Identifier: LGPL-2.1+
+# SPDX-License-Identifier: LGPL-2.1-or-later
 
 from collections.abc import Iterable, Sequence
 
@@ -49,10 +49,6 @@ class Installer(DistributionInstaller):
     def grub_prefix(cls) -> str:
         return "grub2"
 
-    @classmethod
-    def createrepo(cls, context: Context) -> None:
-        Dnf.createrepo(context)
-
     @classmethod
     def dbpath(cls, context: Context) -> str:
         # The Hyperscale SIG uses /usr/lib/sysimage/rpm in its rebuild of rpm for C9S that's shipped in the
@@ -102,8 +98,11 @@ class Installer(DistributionInstaller):
 
     @staticmethod
     def gpgurls(context: Context) -> tuple[str, ...]:
-        keys = ("RPM-GPG-KEY-CentOS-Official", "RPM-GPG-KEY-CentOS-SIG-Extras")
-        return tuple(find_rpm_gpgkey(context, key) or f"https://www.centos.org/keys/{key}" for key in keys)
+        rel = "RPM-GPG-KEY-CentOS-Official" if context.config.release == "9" else "RPM-GPG-KEY-CentOS-Official-SHA256"
+        return tuple(
+            find_rpm_gpgkey(context, key) or f"https://www.centos.org/keys/{key}"
+            for key in (rel, "RPM-GPG-KEY-CentOS-SIG-Extras")
+        )
 
     @classmethod
     def repository_variants(cls, context: Context, repo: str) -> Iterable[RpmRepository]:
@@ -114,12 +113,12 @@ class Installer(DistributionInstaller):
             if repo == "extras":
                 yield RpmRepository(
                     repo.lower(),
-                    f"baseurl={join_mirror(mirror, f'centos-stream/SIGs/$stream/{repo}/$basearch/extras-common')}",
+                    f"baseurl={join_mirror(mirror, f'SIGs/$stream/{repo}/$basearch/extras-common')}",
                     cls.gpgurls(context),
                 )
                 yield RpmRepository(
                     f"{repo.lower()}-source",
-                    f"baseurl={join_mirror(mirror, f'centos-stream/SIGs/$stream/{repo}/source/extras-common')}",
+                    f"baseurl={join_mirror(mirror, f'SIGs/$stream/{repo}/source/extras-common')}",
                     cls.gpgurls(context),
                     enabled=False,
                 )
@@ -127,18 +126,18 @@ class Installer(DistributionInstaller):
             else:
                 yield RpmRepository(
                     repo.lower(),
-                    f"baseurl={join_mirror(mirror, f'centos-stream/$stream/{repo}/$basearch/os')}",
+                    f"baseurl={join_mirror(mirror, f'$stream/{repo}/$basearch/os')}",
                     cls.gpgurls(context),
                 )
                 yield RpmRepository(
                     f"{repo.lower()}-debuginfo",
-                    f"baseurl={join_mirror(mirror, f'centos-stream/$stream/{repo}/$basearch/debug/tree')}",
+                    f"baseurl={join_mirror(mirror, f'$stream/{repo}/$basearch/debug/tree')}",
                     cls.gpgurls(context),
                     enabled=False,
                 )
                 yield RpmRepository(
                     f"{repo.lower()}-source",
-                    f"baseurl={join_mirror(mirror, f'centos-stream/$stream/{repo}/source/tree')}",
+                    f"baseurl={join_mirror(mirror, f'$stream/{repo}/source/tree')}",
                     cls.gpgurls(context),
                     enabled=False,
                 )
@@ -210,21 +209,25 @@ class Installer(DistributionInstaller):
                 ("epel-testing", "epel/testing"),
                 ("epel-next-testing", "epel/testing/next")
             ):
+                # For EPEL we make the assumption that epel is mirrored in the parent directory of the mirror URL and
+                # path we were given. Since this doesn't work for all scenarios, we also allow overriding the mirror
+                # via ane environment variable.
+                url = context.config.environment.get("EPEL_MIRROR", join_mirror(mirror, "../fedora"))
                 yield RpmRepository(
                     repo,
-                    f"baseurl={join_mirror(mirror, f'fedora/{dir}/$releasever/Everything/$basearch')}",
+                    f"baseurl={url}/{dir}/$releasever/Everything/$basearch",
                     gpgurls,
                     enabled=False,
                 )
                 yield RpmRepository(
                     f"{repo}-debuginfo",
-                    f"baseurl={join_mirror(mirror, f'fedora/{dir}/$releasever/Everything/$basearch/debug')}",
+                    f"baseurl={url}/{dir}/$releasever/Everything/$basearch/debug",
                     gpgurls,
                     enabled=False,
                 )
                 yield RpmRepository(
                     f"{repo}-source",
-                    f"baseurl={join_mirror(mirror, f'fedora/{dir}/$releasever/Everything/source/tree')}",
+                    f"baseurl={url}/{dir}/$releasever/Everything/source/tree",
                     gpgurls,
                     enabled=False,
                 )
@@ -302,21 +305,21 @@ class Installer(DistributionInstaller):
                 if mirror := context.config.mirror:
                     yield RpmRepository(
                         f"{sig}-{c}",
-                        f"baseurl={join_mirror(mirror, f'centos-stream/SIGs/$stream/{sig}/$basearch/{c}')}",
+                        f"baseurl={join_mirror(mirror, f'SIGs/$stream/{sig}/$basearch/{c}')}",
                         gpgurls,
                         enabled=False,
                         priority=CENTOS_SIG_REPO_PRIORITY,
                     )
                     yield RpmRepository(
                         f"{sig}-{c}-debuginfo",
-                        f"baseurl={join_mirror(mirror, f'centos-stream/SIGs/$stream/{sig}/$basearch/{c}/debug')}",
+                        f"baseurl={join_mirror(mirror, f'SIGs/$stream/{sig}/$basearch/{c}/debug')}",
                         gpgurls,
                         enabled=False,
                         priority=CENTOS_SIG_REPO_PRIORITY,
                     )
                     yield RpmRepository(
                         f"{sig}-{c}-source",
-                        f"baseurl={join_mirror(mirror, f'centos-stream/SIGs/$stream/{sig}/source/{c}')}",
+                        f"baseurl={join_mirror(mirror, f'SIGs/$stream/{sig}/source/{c}')}",
                         gpgurls,
                         enabled=False,
                         priority=CENTOS_SIG_REPO_PRIORITY,
