@@ -1,4 +1,4 @@
-# SPDX-License-Identifier: LGPL-2.1+
+# SPDX-License-Identifier: LGPL-2.1-or-later
 
 import tempfile
 from collections.abc import Iterable, Sequence
@@ -7,9 +7,9 @@ from pathlib import Path
 from mkosi.archive import extract_tar
 from mkosi.config import Architecture, Config
 from mkosi.context import Context
-from mkosi.distributions import Distribution, DistributionInstaller, PackageType
+from mkosi.distributions import DistributionInstaller, PackageType
 from mkosi.installer import PackageManager
-from mkosi.installer.apt import Apt
+from mkosi.installer.apt import Apt, AptRepository
 from mkosi.log import die
 from mkosi.run import run
 from mkosi.sandbox import Mount
@@ -33,22 +33,18 @@ class Installer(DistributionInstaller):
     def default_release(cls) -> str:
         return "testing"
 
-    @classmethod
-    def default_tools_tree_distribution(cls) -> Distribution:
-        return Distribution.debian
-
     @classmethod
     def package_manager(cls, config: Config) -> type[PackageManager]:
         return Apt
 
     @staticmethod
     @listify
-    def repositories(context: Context, local: bool = True) -> Iterable[Apt.Repository]:
+    def repositories(context: Context, local: bool = True) -> Iterable[AptRepository]:
         types = ("deb", "deb-src")
         components = ("main", *context.config.repositories)
 
         if context.config.local_mirror and local:
-            yield Apt.Repository(
+            yield AptRepository(
                 types=("deb",),
                 url=context.config.local_mirror,
                 suite=context.config.release,
@@ -60,7 +56,7 @@ class Installer(DistributionInstaller):
         mirror = context.config.mirror or "http://deb.debian.org/debian"
         signedby = Path("/usr/share/keyrings/debian-archive-keyring.gpg")
 
-        yield Apt.Repository(
+        yield AptRepository(
             types=types,
             url=mirror,
             suite=context.config.release,
@@ -71,7 +67,7 @@ class Installer(DistributionInstaller):
         # Debug repos are typically not mirrored.
         url = "http://deb.debian.org/debian-debug"
 
-        yield Apt.Repository(
+        yield AptRepository(
             types=types,
             url=url,
             suite=f"{context.config.release}-debug",
@@ -82,7 +78,7 @@ class Installer(DistributionInstaller):
         if context.config.release in ("unstable", "sid"):
             return
 
-        yield Apt.Repository(
+        yield AptRepository(
             types=types,
             url=mirror,
             suite=f"{context.config.release}-updates",
@@ -90,7 +86,7 @@ class Installer(DistributionInstaller):
             signedby=signedby,
         )
 
-        yield Apt.Repository(
+        yield AptRepository(
             types=types,
             # Security updates repos are never mirrored.
             url="http://security.debian.org/debian-security",
@@ -103,10 +99,6 @@ class Installer(DistributionInstaller):
     def setup(cls, context: Context) -> None:
         Apt.setup(context, cls.repositories(context))
 
-    @classmethod
-    def createrepo(cls, context: Context) -> None:
-        Apt.createrepo(context)
-
     @classmethod
     def install(cls, context: Context) -> None:
         # Instead of using debootstrap, we replicate its core functionality here. Because dpkg does not have
@@ -165,12 +157,12 @@ class Installer(DistributionInstaller):
 
         for deb in essential:
             # If a deb path is in the form of "/var/cache/apt/<deb>", we transform it to the corresponding path in
-            # mkosi's package cache directory. If it's relative to /work/packages, we transform it to the corresponding
+            # mkosi's package cache directory. If it's relative to /repository, we transform it to the corresponding
             # path in mkosi's local package repository. Otherwise, we use the path as is.
             if Path(deb).is_relative_to("/var/cache"):
                 path = context.config.package_cache_dir_or_default() / Path(deb).relative_to("/var")
-            elif Path(deb).is_relative_to("/work/packages"):
-                path = context.packages / Path(deb).relative_to("/work/packages")
+            elif Path(deb).is_relative_to("/repository"):
+                path = context.repository / Path(deb).relative_to("/repository")
             else:
                 path = Path(deb)
 
@@ -181,7 +173,17 @@ class Installer(DistributionInstaller):
                     stdout=o,
                     sandbox=context.sandbox(binary="dpkg-deb"),
                 )
-                extract_tar(Path(o.name), context.root, log=False, sandbox=context.sandbox)
+                extract_tar(
+                    Path(o.name),
+                    context.root,
+                    log=False,
+                    options=(
+                        [f"--exclude=./{glob}" for glob in Apt.documentation_exclude_globs]
+                        if not context.config.with_docs
+                        else []
+                    ),
+                    sandbox=context.sandbox
+                )
 
         # Finally, run apt to properly install packages in the chroot without having to worry that maintainer
         # scripts won't find basic tools that they depend on.
@@ -196,6 +198,8 @@ class Installer(DistributionInstaller):
         # Note: despite writing in /usr/sbin, this file is not shipped by the OS and instead should be managed by
         # the admin.
         policyrcd = context.root / "usr/sbin/policy-rc.d"
+        with umask(~0o755):
+            policyrcd.parent.mkdir(parents=True, exist_ok=True)
         with umask(~0o644):
             policyrcd.write_text("#!/bin/sh\nexit 101\n")
 
@@ -240,7 +244,7 @@ class Installer(DistributionInstaller):
         return a
 
 
-def install_apt_sources(context: Context, repos: Iterable[Apt.Repository]) -> None:
+def install_apt_sources(context: Context, repos: Iterable[AptRepository]) -> None:
     if not (context.root / "usr/bin/apt").exists():
         return
 
