@@ -1,4 +1,4 @@
-# SPDX-License-Identifier: LGPL-2.1+
+# SPDX-License-Identifier: LGPL-2.1-or-later
 
 import re
 from collections.abc import Iterable, Sequence
@@ -7,7 +7,6 @@ from pathlib import Path
 from mkosi.config import Architecture, Config
 from mkosi.context import Context
 from mkosi.distributions import (
-    Distribution,
     DistributionInstaller,
     PackageType,
     join_mirror,
@@ -62,10 +61,6 @@ class Installer(DistributionInstaller):
     def default_release(cls) -> str:
         return "40"
 
-    @classmethod
-    def default_tools_tree_distribution(cls) -> Distribution:
-        return Distribution.fedora
-
     @classmethod
     def grub_prefix(cls) -> str:
         return "grub2"
@@ -74,10 +69,6 @@ class Installer(DistributionInstaller):
     def package_manager(cls, config: Config) -> type[PackageManager]:
         return Dnf
 
-    @classmethod
-    def createrepo(cls, context: Context) -> None:
-        Dnf.createrepo(context)
-
     @classmethod
     def setup(cls, context: Context) -> None:
         Dnf.setup(context, cls.repositories(context), filelists=False)
@@ -113,18 +104,18 @@ class Installer(DistributionInstaller):
                 yield RpmRepository(repo.lower(), f"{url}/source/tree", gpgurls, enabled=False)
         elif (m := context.config.mirror):
             directory = "development" if context.config.release == "rawhide" else "releases"
-            url = f"baseurl={join_mirror(m, f'fedora/linux/{directory}/$releasever/Everything')}"
+            url = f"baseurl={join_mirror(m, f'linux/{directory}/$releasever/Everything')}"
             yield RpmRepository("fedora", f"{url}/$basearch/os", gpgurls)
             yield RpmRepository("fedora-debuginfo", f"{url}/$basearch/debug/tree", gpgurls, enabled=False)
             yield RpmRepository("fedora-source", f"{url}/source/tree", gpgurls, enabled=False)
 
             if context.config.release != "rawhide":
-                url = f"baseurl={join_mirror(m, 'fedora/linux/updates/$releasever/Everything')}"
+                url = f"baseurl={join_mirror(m, 'linux/updates/$releasever/Everything')}"
                 yield RpmRepository("updates", f"{url}/$basearch", gpgurls)
                 yield RpmRepository("updates-debuginfo", f"{url}/$basearch/debug", gpgurls, enabled=False)
                 yield RpmRepository("updates-source", f"{url}/source/tree", gpgurls, enabled=False)
 
-                url = f"baseurl={join_mirror(m, 'fedora/linux/updates/testing/$releasever/Everything')}"
+                url = f"baseurl={join_mirror(m, 'linux/updates/testing/$releasever/Everything')}"
                 yield RpmRepository("updates-testing", f"{url}/$basearch", gpgurls, enabled=False)
                 yield RpmRepository("updates-testing-debuginfo", f"{url}/$basearch/debug", gpgurls, enabled=False)
                 yield RpmRepository("updates-testing-source", f"{url}/source/tree", gpgurls, enabled=False)
