@@ -1,13 +1,13 @@
-# SPDX-License-Identifier: LGPL-2.1+
+# SPDX-License-Identifier: LGPL-2.1-or-later
 
 import tempfile
-import xml.etree.ElementTree as ElementTree
 from collections.abc import Iterable, Sequence
 from pathlib import Path
+from xml.etree import ElementTree
 
 from mkosi.config import Architecture, Config
 from mkosi.context import Context
-from mkosi.distributions import Distribution, DistributionInstaller, PackageType
+from mkosi.distributions import DistributionInstaller, PackageType, join_mirror
 from mkosi.installer import PackageManager
 from mkosi.installer.dnf import Dnf
 from mkosi.installer.rpm import RpmRepository, find_rpm_gpgkey, setup_rpm
@@ -36,10 +36,6 @@ class Installer(DistributionInstaller):
     def default_release(cls) -> str:
         return "tumbleweed"
 
-    @classmethod
-    def default_tools_tree_distribution(cls) -> Distribution:
-        return Distribution.opensuse
-
     @classmethod
     def grub_prefix(cls) -> str:
         return "grub2"
@@ -51,13 +47,6 @@ class Installer(DistributionInstaller):
         else:
             return Dnf
 
-    @classmethod
-    def createrepo(cls, context: Context) -> None:
-        if context.config.find_binary("zypper"):
-            Zypper.createrepo(context)
-        else:
-            Dnf.createrepo(context)
-
     @classmethod
     def setup(cls, context: Context) -> None:
         zypper = context.config.find_binary("zypper")
@@ -97,54 +86,131 @@ class Installer(DistributionInstaller):
     @classmethod
     @listify
     def repositories(cls, context: Context) -> Iterable[RpmRepository]:
+        if context.config.local_mirror:
+            yield RpmRepository(id="local-mirror", url=f"baseurl={context.config.local_mirror}", gpgurls=())
+            return
+
         zypper = context.config.find_binary("zypper")
-
-        release = context.config.release
-        if release == "leap":
-            release = "stable"
-
         mirror = context.config.mirror or "https://download.opensuse.org"
 
-        # If the release looks like a timestamp, it's Tumbleweed. 13.x is legacy
-        # (14.x won't ever appear). For anything else, let's default to Leap.
-        if context.config.local_mirror:
-            release_url = f"{context.config.local_mirror}"
-            updates_url = None
-        if release.isdigit() or release == "tumbleweed":
-            release_url = f"{mirror}/tumbleweed/repo/oss/"
-            updates_url = f"{mirror}/update/tumbleweed/"
+        if context.config.release == "tumbleweed" or context.config.release.isdigit():
             gpgurls = (
                 *([p] if (p := find_rpm_gpgkey(context, key="RPM-GPG-KEY-openSUSE-Tumbleweed")) else []),
                 *([p] if (p := find_rpm_gpgkey(context, key="RPM-GPG-KEY-openSUSE")) else []),
             )
-        elif release in ("current", "stable"):
-            release_url = f"{mirror}/distribution/openSUSE-{release}/repo/oss/"
-            updates_url = f"{mirror}/update/openSUSE-{release}/"
-            gpgurls=()
-        else:
-            release_url = f"{mirror}/distribution/leap/{release}/repo/oss/"
-            updates_url = f"{mirror}/update/leap/{release}/oss/"
-            gpgurls=()
 
-        if context.config.local_mirror:
-            yield RpmRepository(id="local-mirror", url=f"baseurl={context.config.local_mirror}", gpgurls=())
-        else:
-            yield RpmRepository(
-                id="repo-oss",
-                url=f"baseurl={release_url}",
-                gpgurls=gpgurls or (fetch_gpgurls(context, release_url) if not zypper else ()),
-            )
-            if updates_url is not None:
+            if context.config.release == "tumbleweed":
+                if context.config.architecture == Architecture.x86_64:
+                    subdir = ""
+                else:
+                    subdir = f"ports/{cls.architecture(context.config.architecture)}"
+            else:
+                if context.config.architecture != Architecture.x86_64:
+                    die(f"Old snapshots are only supported for x86-64 on {cls.pretty_name()}")
+
+                subdir = f"history/{context.config.release}"
+
+            for repo in ("oss", "non-oss"):
+                url = join_mirror(mirror, f"{subdir}/tumbleweed/repo/{repo}")
                 yield RpmRepository(
-                    id="repo-update",
-                    url=f"baseurl={updates_url}",
-                    gpgurls=gpgurls or (fetch_gpgurls(context, updates_url) if not zypper else ()),
+                    id=repo,
+                    url=f"baseurl={url}",
+                    gpgurls=gpgurls or (fetch_gpgurls(context, url) if not zypper else ()),
+                    enabled=repo == "oss",
                 )
 
+                if context.config.release == "tumbleweed":
+                    for d in ("debug", "source"):
+                        url = join_mirror(mirror, f"{subdir}/{d}/tumbleweed/repo/{repo}")
+                        yield RpmRepository(
+                            id=f"{repo}-{d}",
+                            url=f"baseurl={url}",
+                            gpgurls=gpgurls or (fetch_gpgurls(context, url) if not zypper else ()),
+                            enabled=False,
+                        )
+
+            if context.config.release == "tumbleweed":
+                url = join_mirror(mirror, f"{subdir}/update/tumbleweed")
+                yield RpmRepository(
+                    id="oss-update",
+                    url=f"baseurl={url}",
+                    gpgurls=gpgurls or (fetch_gpgurls(context, url) if not zypper else ()),
+                )
+
+                url = join_mirror(mirror, f"{subdir}/update/tumbleweed-non-oss")
+                yield RpmRepository(
+                    id="non-oss-update",
+                    url=f"baseurl={url}",
+                    gpgurls=gpgurls or (fetch_gpgurls(context, url) if not zypper else ()),
+                    enabled=False,
+                )
+        else:
+            if (
+                context.config.release in ("current", "stable", "leap") and
+                context.config.architecture != Architecture.x86_64
+            ):
+                die(f"{cls.pretty_name()} only supports current and stable releases for the x86-64 architecture",
+                    hint="Specify either tumbleweed or a specific leap release such as 15.6")
+
+            if context.config.release in ("current", "stable", "leap"):
+                release = "openSUSE-current"
+            else:
+                release = f"leap/{context.config.release}"
+
+            if context.config.architecture == Architecture.x86_64:
+                subdir = ""
+            else:
+                subdir = f"ports/{cls.architecture(context.config.architecture)}"
+
+            for repo in ("oss", "non-oss"):
+                url = join_mirror(mirror, f"{subdir}/distribution/{release}/repo/{repo}")
+                yield RpmRepository(
+                    id=repo,
+                    url=f"baseurl={url}",
+                    gpgurls=fetch_gpgurls(context, url) if not zypper else (),
+                    enabled=repo == "oss",
+                )
+
+            for d in ("debug", "source"):
+                for repo in ("oss", "non-oss"):
+                    url = join_mirror(mirror, f"{subdir}/{d}/distribution/{release}/repo/{repo}")
+                    yield RpmRepository(
+                        id=f"{repo}-{d}",
+                        url=f"baseurl={url}",
+                        gpgurls=fetch_gpgurls(context, url) if not zypper else (),
+                        enabled=False,
+                    )
+
+            if context.config.release in ("current", "stable", "leap"):
+                url = join_mirror(mirror, f"{subdir}/update/openSUSE-current")
+                yield RpmRepository(
+                    id="oss-update",
+                    url=f"baseurl={url}",
+                    gpgurls=fetch_gpgurls(context, url) if not zypper else (),
+                )
+
+                url = join_mirror(mirror, f"{subdir}/update/openSUSE-non-oss-current")
+                yield RpmRepository(
+                    id="non-oss-update",
+                    url=f"baseurl={url}",
+                    gpgurls=fetch_gpgurls(context, url) if not zypper else (),
+                    enabled=False,
+                )
+            else:
+                for repo in ("oss", "non-oss"):
+                    url = join_mirror(mirror, f"{subdir}/update/{release}/{repo}")
+                    yield RpmRepository(
+                        id=f"{repo}-update",
+                        url=f"baseurl={url}",
+                        gpgurls=fetch_gpgurls(context, url) if not zypper else (),
+                        enabled=repo == "oss",
+                    )
+
     @classmethod
     def architecture(cls, arch: Architecture) -> str:
         a = {
             Architecture.x86_64 : "x86_64",
+            Architecture.arm64  : "aarch64",
         }.get(arch)
 
         if not a:
