@@ -1,11 +1,11 @@
-# SPDX-License-Identifier: LGPL-2.1+
+# SPDX-License-Identifier: LGPL-2.1-or-later
 
 from collections.abc import Iterable
 from pathlib import Path
 
 from mkosi.context import Context
-from mkosi.distributions import Distribution, debian
-from mkosi.installer.apt import Apt
+from mkosi.distributions import debian
+from mkosi.installer.apt import AptRepository
 from mkosi.util import listify
 
 
@@ -18,13 +18,9 @@ class Installer(debian.Installer):
     def default_release(cls) -> str:
         return "noble"
 
-    @classmethod
-    def default_tools_tree_distribution(cls) -> Distribution:
-        return Distribution.ubuntu
-
     @staticmethod
     @listify
-    def repositories(context: Context, local: bool = True) -> Iterable[Apt.Repository]:
+    def repositories(context: Context, local: bool = True) -> Iterable[AptRepository]:
         types = ("deb", "deb-src")
 
         # From kinetic onwards, the usr-is-merged package is available in universe and is required by
@@ -33,7 +29,7 @@ class Installer(debian.Installer):
         components = (*components, *context.config.repositories)
 
         if context.config.local_mirror and local:
-            yield Apt.Repository(
+            yield AptRepository(
                 types=("deb",),
                 url=context.config.local_mirror,
                 suite=context.config.release,
@@ -49,7 +45,7 @@ class Installer(debian.Installer):
 
         signedby = Path("/usr/share/keyrings/ubuntu-archive-keyring.gpg")
 
-        yield Apt.Repository(
+        yield AptRepository(
             types=types,
             url=mirror,
             suite=context.config.release,
@@ -57,7 +53,7 @@ class Installer(debian.Installer):
             signedby=signedby,
         )
 
-        yield Apt.Repository(
+        yield AptRepository(
             types=types,
             url=mirror,
             suite=f"{context.config.release}-updates",
@@ -67,11 +63,11 @@ class Installer(debian.Installer):
 
         # Security updates repos are never mirrored. But !x86 are on the ports server.
         if context.config.architecture.is_x86_variant():
-            mirror = "http://security.ubuntu.com/ubuntu/"
+            mirror = "http://security.ubuntu.com/ubuntu"
         else:
-            mirror = "http://ports.ubuntu.com/"
+            mirror = "http://ports.ubuntu.com"
 
-        yield Apt.Repository(
+        yield AptRepository(
             types=types,
             url=mirror,
             suite=f"{context.config.release}-security",
