@@ -1,4 +1,4 @@
-# SPDX-License-Identifier: LGPL-2.1+
+# SPDX-License-Identifier: LGPL-2.1-or-later
 
 from pathlib import Path
 
@@ -25,6 +25,10 @@ class PackageManager:
     def cache_subdirs(cls, cache: Path) -> list[Path]:
         return []
 
+    @classmethod
+    def state_subdirs(cls, state: Path) -> list[Path]:
+        return  []
+
     @classmethod
     def scripts(cls, context: Context) -> dict[str, list[PathString]]:
         return {}
@@ -54,7 +58,7 @@ class PackageManager:
     def mounts(cls, context: Context) -> list[Mount]:
         mounts = [
             *finalize_crypto_mounts(context.config),
-            Mount(context.packages, "/work/packages"),
+            Mount(context.repository, "/repository"),
         ]
 
         if context.config.local_mirror and (mirror := startswith(context.config.local_mirror, "file://")):
@@ -84,7 +88,11 @@ class PackageManager:
         return mounts
 
     @classmethod
-    def sync(cls, context: Context) -> None:
+    def sync(cls, context: Context, force: bool) -> None:
+        pass
+
+    @classmethod
+    def createrepo(cls, context: Context) -> None:
         pass
 
 
