@@ -1,40 +1,50 @@
-# SPDX-License-Identifier: LGPL-2.1+
-import os
+# SPDX-License-Identifier: LGPL-2.1-or-later
+
+import dataclasses
 import textwrap
 from collections.abc import Iterable, Sequence
 from pathlib import Path
-from typing import NamedTuple, Optional
+from typing import Optional
 
-from mkosi.config import Config, ConfigFeature
+from mkosi.config import PACKAGE_GLOBS, Config, ConfigFeature
 from mkosi.context import Context
 from mkosi.installer import PackageManager
 from mkosi.log import die
-from mkosi.mounts import finalize_source_mounts
 from mkosi.run import run
 from mkosi.sandbox import Mount, apivfs_cmd
 from mkosi.types import _FILE, CompletedProcess, PathString
 from mkosi.util import umask
 
 
+@dataclasses.dataclass(frozen=True)
+class AptRepository:
+    types: tuple[str, ...]
+    url: str
+    suite: str
+    components: tuple[str, ...]
+    signedby: Optional[Path]
+
+    def __str__(self) -> str:
+        return textwrap.dedent(
+            f"""\
+            Types: {" ".join(self.types)}
+            URIs: {self.url}
+            Suites: {self.suite}
+            Components: {" ".join(self.components)}
+            {"Signed-By" if self.signedby else "Trusted"}: {self.signedby or "yes"}
+
+            """
+        )
+
+
 class Apt(PackageManager):
-    class Repository(NamedTuple):
-        types: tuple[str, ...]
-        url: str
-        suite: str
-        components: tuple[str, ...]
-        signedby: Optional[Path]
-
-        def __str__(self) -> str:
-            return textwrap.dedent(
-                f"""\
-                Types: {" ".join(self.types)}
-                URIs: {self.url}
-                Suites: {self.suite}
-                Components: {" ".join(self.components)}
-                {"Signed-By" if self.signedby else "Trusted"}: {self.signedby or "yes"}
-
-                """
-            )
+    documentation_exclude_globs = [
+        "usr/share/doc/*",
+        "usr/share/man/*",
+        "usr/share/groff/*",
+        "usr/share/gtk-doc/*",
+        "usr/share/info/*",
+    ]
 
     @classmethod
     def executable(cls, config: Config) -> str:
@@ -85,7 +95,7 @@ class Apt(PackageManager):
         }
 
     @classmethod
-    def setup(cls, context: Context, repos: Iterable[Repository]) -> None:
+    def setup(cls, context: Context, repos: Iterable[AptRepository]) -> None:
         (context.pkgmngr / "etc/apt").mkdir(exist_ok=True, parents=True)
         (context.pkgmngr / "etc/apt/apt.conf.d").mkdir(exist_ok=True, parents=True)
         (context.pkgmngr / "etc/apt/preferences.d").mkdir(exist_ok=True, parents=True)
@@ -179,14 +189,8 @@ class Apt(PackageManager):
             ]
 
         if not context.config.with_docs:
-            cmdline += [
-                "-o", "DPkg::Options::=--path-exclude=/usr/share/doc/*",
-                "-o", "DPkg::Options::=--path-include=/usr/share/doc/*/copyright",
-                "-o", "DPkg::Options::=--path-exclude=/usr/share/man/*",
-                "-o", "DPkg::Options::=--path-exclude=/usr/share/groff/*",
-                "-o", "DPkg::Options::=--path-exclude=/usr/share/gtk-doc/*",
-                "-o", "DPkg::Options::=--path-exclude=/usr/share/info/*",
-            ]
+            cmdline += [f"--option=DPkg::Options::=--path-exclude=/{glob}" for glob in cls.documentation_exclude_globs]
+            cmdline += ["--option=DPkg::Options::=--path-include=/usr/share/doc/*/copyright"]
 
         if context.config.proxy_url:
             cmdline += [
@@ -207,32 +211,28 @@ class Apt(PackageManager):
         mounts: Sequence[Mount] = (),
         stdout: _FILE = None,
     ) -> CompletedProcess:
-        with finalize_source_mounts(
-            context.config,
-            ephemeral=os.getuid() == 0 and context.config.build_sources_ephemeral,
-        ) as sources:
-            return run(
-                cls.cmd(context, "apt-get") + [operation, *arguments],
-                sandbox=(
-                    context.sandbox(
-                        binary="apt-get",
-                        network=True,
-                        mounts=[Mount(context.root, "/buildroot"), *cls.mounts(context), *sources, *mounts],
-                        options=["--dir", "/work/src", "--chdir", "/work/src"],
-                        extra=apivfs_cmd() if apivfs else []
-                    )
-                ),
-                env=context.config.environment | cls.finalize_environment(context),
-                stdout=stdout,
-            )
+        return run(
+            cls.cmd(context, "apt-get") + [operation, *arguments],
+            sandbox=(
+                context.sandbox(
+                    binary="apt-get",
+                    network=True,
+                    vartmp=True,
+                    mounts=[Mount(context.root, "/buildroot"), *cls.mounts(context), *mounts],
+                    extra=apivfs_cmd() if apivfs else []
+                )
+            ),
+            env=context.config.environment | cls.finalize_environment(context),
+            stdout=stdout,
+        )
 
     @classmethod
-    def sync(cls, context: Context) -> None:
+    def sync(cls, context: Context, force: bool) -> None:
         cls.invoke(context, "update")
 
     @classmethod
     def createrepo(cls, context: Context) -> None:
-        if not (conf := context.packages / "conf/distributions").exists():
+        if not (conf := context.repository / "conf/distributions").exists():
             conf.parent.mkdir(exist_ok=True)
             conf.write_text(
                 textwrap.dedent(
@@ -253,13 +253,12 @@ class Apt(PackageManager):
                 "--ignore=extension",
                 "includedeb",
                 "mkosi",
-                *(d.name for d in context.packages.glob("*.deb")),
-                *(d.name for d in context.packages.glob("*.ddeb")),
+                *(d.name for glob in PACKAGE_GLOBS for d in context.repository.glob(glob) if "deb" in glob),
             ],
             sandbox=context.sandbox(
                 binary="reprepro",
-                mounts=[Mount(context.packages, context.packages)],
-                options=["--chdir", context.packages],
+                mounts=[Mount(context.repository, context.repository)],
+                options=["--chdir", context.repository],
             ),
         )
 
@@ -269,7 +268,7 @@ class Apt(PackageManager):
                 """\
                 Enabled: yes
                 Types: deb
-                URIs: file:///work/packages
+                URIs: file:///repository
                 Suites: mkosi
                 Components: main
                 Trusted: yes
