@@ -1,5 +1,4 @@
-# SPDX-License-Identifier: LGPL-2.1+
-import os
+# SPDX-License-Identifier: LGPL-2.1-or-later
 import textwrap
 from collections.abc import Iterable, Sequence
 from pathlib import Path
@@ -9,7 +8,6 @@ from mkosi.context import Context
 from mkosi.installer import PackageManager
 from mkosi.installer.rpm import RpmRepository, rpm_cmd
 from mkosi.log import ARG_DEBUG
-from mkosi.mounts import finalize_source_mounts
 from mkosi.run import run
 from mkosi.sandbox import Mount, apivfs_cmd
 from mkosi.types import _FILE, CompletedProcess, PathString
@@ -107,6 +105,12 @@ class Dnf(PackageManager):
 
                     f.write("\n")
 
+    @classmethod
+    def finalize_environment(cls, context: Context) -> dict[str, str]:
+        return super().finalize_environment(context) | {
+            "RPM_FORCE_DEBIAN": "1",
+        }
+
     @classmethod
     def cmd(
             cls,
@@ -188,24 +192,20 @@ class Dnf(PackageManager):
         cached_metadata: bool = True,
     ) -> CompletedProcess:
         try:
-            with finalize_source_mounts(
-                context.config,
-                ephemeral=os.getuid() == 0 and context.config.build_sources_ephemeral,
-            ) as sources:
-                return run(
-                    cls.cmd(context, cached_metadata=cached_metadata) + [operation, *arguments],
-                    sandbox=(
-                        context.sandbox(
-                            binary=cls.executable(context.config),
-                            network=True,
-                            mounts=[Mount(context.root, "/buildroot"), *cls.mounts(context), *sources],
-                            options=["--dir", "/work/src", "--chdir", "/work/src"],
-                            extra=apivfs_cmd() if apivfs else [],
-                        )
-                    ),
-                    env=context.config.environment | cls.finalize_environment(context),
-                    stdout=stdout,
-                )
+            return run(
+                cls.cmd(context, cached_metadata=cached_metadata) + [operation, *arguments],
+                sandbox=(
+                    context.sandbox(
+                        binary=cls.executable(context.config),
+                        network=True,
+                        vartmp=True,
+                        mounts=[Mount(context.root, "/buildroot"), *cls.mounts(context)],
+                        extra=apivfs_cmd() if apivfs else [],
+                    )
+                ),
+                env=context.config.environment | cls.finalize_environment(context),
+                stdout=stdout,
+            )
         finally:
             # dnf interprets the log directory relative to the install root so there's nothing we can do but to remove
             # the log files from the install root afterwards.
@@ -215,28 +215,25 @@ class Dnf(PackageManager):
                         p.unlink()
 
     @classmethod
-    def sync(cls, context: Context, options: Sequence[str] = ()) -> None:
+    def sync(cls, context: Context, force: bool, arguments: Sequence[str] = ()) -> None:
         cls.invoke(
             context,
             "makecache",
-            arguments=[
-                *(["--refresh"] if context.args.force > 1 or context.config.cacheonly == Cacheonly.never else []),
-                *options,
-            ],
+            arguments=[*(["--refresh"] if force else []), *arguments],
             cached_metadata=False,
         )
 
     @classmethod
     def createrepo(cls, context: Context) -> None:
-        run(["createrepo_c", context.packages],
-            sandbox=context.sandbox(binary="createrepo_c", mounts=[Mount(context.packages, context.packages)]))
+        run(["createrepo_c", context.repository],
+            sandbox=context.sandbox(binary="createrepo_c", mounts=[Mount(context.repository, context.repository)]))
 
         (context.pkgmngr / "etc/yum.repos.d/mkosi-local.repo").write_text(
             textwrap.dedent(
                 """\
                 [mkosi]
                 name=mkosi
-                baseurl=file:///work/packages
+                baseurl=file:///repository
                 gpgcheck=0
                 metadata_expire=never
                 priority=10
@@ -244,4 +241,4 @@ class Dnf(PackageManager):
             )
         )
 
-        cls.sync(context, options=["--disablerepo=*", "--enablerepo=mkosi"])
+        cls.sync(context, force=True, arguments=["--disablerepo=*", "--enablerepo=mkosi"])
