@@ -1,15 +1,14 @@
-# SPDX-License-Identifier: LGPL-2.1+
-import os
+# SPDX-License-Identifier: LGPL-2.1-or-later
+
+import dataclasses
 import shutil
 import textwrap
 from collections.abc import Iterable, Sequence
 from pathlib import Path
-from typing import NamedTuple
 
-from mkosi.config import Cacheonly, Config
+from mkosi.config import Config
 from mkosi.context import Context
 from mkosi.installer import PackageManager
-from mkosi.mounts import finalize_source_mounts
 from mkosi.run import run
 from mkosi.sandbox import Mount, apivfs_cmd
 from mkosi.types import _FILE, CompletedProcess, PathString
@@ -17,11 +16,13 @@ from mkosi.util import umask
 from mkosi.versioncomp import GenericVersion
 
 
-class Pacman(PackageManager):
-    class Repository(NamedTuple):
-        id: str
-        url: str
+@dataclasses.dataclass(frozen=True)
+class PacmanRepository:
+    id: str
+    url: str
 
+
+class Pacman(PackageManager):
     @classmethod
     def executable(cls, config: Config) -> str:
         return "pacman"
@@ -34,6 +35,10 @@ class Pacman(PackageManager):
     def cache_subdirs(cls, cache: Path) -> list[Path]:
         return [cache / "pkg"]
 
+    @classmethod
+    def state_subdirs(cls, state: Path) -> list[Path]:
+        return [state / "local"]
+
     @classmethod
     def scripts(cls, context: Context) -> dict[str, list[PathString]]:
         return {
@@ -50,7 +55,7 @@ class Pacman(PackageManager):
             *super().mounts(context),
             # pacman writes downloaded packages to the first writable cache directory. We don't want it to write to our
             # local repository directory so we expose it as a read-only directory to pacman.
-            Mount(context.packages, "/var/cache/pacman/mkosi", ro=True),
+            Mount(context.repository, "/var/cache/pacman/mkosi", ro=True),
         ]
 
         if (context.root / "var/lib/pacman/local").exists():
@@ -62,7 +67,7 @@ class Pacman(PackageManager):
         return mounts
 
     @classmethod
-    def setup(cls, context: Context, repositories: Iterable[Repository]) -> None:
+    def setup(cls, context: Context, repositories: Iterable[PacmanRepository]) -> None:
         if context.config.repository_key_check:
             sig_level = "Required DatabaseOptional"
         else:
@@ -159,34 +164,24 @@ class Pacman(PackageManager):
         apivfs: bool = False,
         stdout: _FILE = None,
     ) -> CompletedProcess:
-        with finalize_source_mounts(
-            context.config,
-            ephemeral=os.getuid() == 0 and context.config.build_sources_ephemeral,
-        ) as sources:
-            return run(
-                cls.cmd(context) + [operation, *arguments],
-                sandbox=(
-                    context.sandbox(
-                        binary="pacman",
-                        network=True,
-                        mounts=[Mount(context.root, "/buildroot"), *cls.mounts(context), *sources],
-                        options=["--dir", "/work/src", "--chdir", "/work/src"],
-                        extra=apivfs_cmd() if apivfs else [],
-                    )
-                ),
-                env=context.config.environment | cls.finalize_environment(context),
-                stdout=stdout,
-            )
+        return run(
+            cls.cmd(context) + [operation, *arguments],
+            sandbox=(
+                context.sandbox(
+                    binary="pacman",
+                    network=True,
+                    vartmp=True,
+                    mounts=[Mount(context.root, "/buildroot"), *cls.mounts(context)],
+                    extra=apivfs_cmd() if apivfs else [],
+                )
+            ),
+            env=context.config.environment | cls.finalize_environment(context),
+            stdout=stdout,
+        )
 
     @classmethod
-    def sync(cls, context: Context) -> None:
-        cls.invoke(
-            context,
-            "--sync",
-            ["--refresh"] + (
-                ["--refresh"] if context.args.force > 1 or context.config.cacheonly == Cacheonly.never else []
-            )
-        )
+    def sync(cls, context: Context, force: bool) -> None:
+        cls.invoke(context, "--sync", ["--refresh", *(["--refresh"] if force else [])])
 
     @classmethod
     def createrepo(cls, context: Context) -> None:
@@ -194,10 +189,10 @@ class Pacman(PackageManager):
             [
                 "repo-add",
                 "--quiet",
-                context.packages / "mkosi.db.tar",
-                *sorted(context.packages.glob("*.pkg.tar*"), key=lambda p: GenericVersion(Path(p).name))
+                context.repository / "mkosi.db.tar",
+                *sorted(context.repository.glob("*.pkg.tar*"), key=lambda p: GenericVersion(Path(p).name))
             ],
-            sandbox=context.sandbox(binary="repo-add", mounts=[Mount(context.packages, context.packages)]),
+            sandbox=context.sandbox(binary="repo-add", mounts=[Mount(context.repository, context.repository)]),
         )
 
         (context.pkgmngr / "etc/mkosi-local.conf").write_text(
@@ -213,6 +208,6 @@ class Pacman(PackageManager):
 
         # pacman can't sync a single repository, so we go behind its back and do it ourselves.
         shutil.move(
-            context.packages / "mkosi.db.tar",
+            context.repository / "mkosi.db.tar",
             context.package_cache_dir / "lib/pacman/sync/mkosi.db"
         )
