@@ -1,15 +1,13 @@
-# SPDX-License-Identifier: LGPL-2.1+
+# SPDX-License-Identifier: LGPL-2.1-or-later
 import hashlib
-import os
 import textwrap
 from collections.abc import Iterable, Sequence
 from pathlib import Path
 
-from mkosi.config import Cacheonly, Config, yes_no
+from mkosi.config import Config, yes_no
 from mkosi.context import Context
 from mkosi.installer import PackageManager
 from mkosi.installer.rpm import RpmRepository, rpm_cmd
-from mkosi.mounts import finalize_source_mounts
 from mkosi.run import run
 from mkosi.sandbox import Mount, apivfs_cmd
 from mkosi.types import _FILE, CompletedProcess, PathString
@@ -100,7 +98,10 @@ class Zypper(PackageManager):
 
     @classmethod
     def finalize_environment(cls, context: Context) -> dict[str, str]:
-        return super().finalize_environment(context) | {"ZYPP_CONF": "/etc/zypp/zypp.conf"}
+        return super().finalize_environment(context) | {
+            "ZYPP_CONF": "/etc/zypp/zypp.conf",
+            "RPM_FORCE_DEBIAN": "1",
+        }
 
     @classmethod
     def cmd(cls, context: Context) -> list[PathString]:
@@ -124,44 +125,36 @@ class Zypper(PackageManager):
         apivfs: bool = False,
         stdout: _FILE = None,
     ) -> CompletedProcess:
-        with finalize_source_mounts(
-            context.config,
-            ephemeral=os.getuid() == 0 and context.config.build_sources_ephemeral,
-        ) as sources:
-            return run(
-                cls.cmd(context) + [operation, *arguments],
-                sandbox=(
-                    context.sandbox(
-                        binary="zypper",
-                        network=True,
-                        mounts=[Mount(context.root, "/buildroot"), *cls.mounts(context), *sources],
-                        options=["--dir", "/work/src", "--chdir", "/work/src"],
-                        extra=apivfs_cmd() if apivfs else [],
-                    )
-                ),
-                env=context.config.environment | cls.finalize_environment(context),
-                stdout=stdout,
-            )
-
-    @classmethod
-    def sync(cls, context: Context) -> None:
-        cls.invoke(
-            context,
-            "refresh",
-            ["--force"] if context.args.force > 1 or context.config.cacheonly == Cacheonly.never else []
+        return run(
+            cls.cmd(context) + [operation, *arguments],
+            sandbox=(
+                context.sandbox(
+                    binary="zypper",
+                    network=True,
+                    vartmp=True,
+                    mounts=[Mount(context.root, "/buildroot"), *cls.mounts(context)],
+                    extra=apivfs_cmd() if apivfs else [],
+                )
+            ),
+            env=context.config.environment | cls.finalize_environment(context),
+            stdout=stdout,
         )
 
+    @classmethod
+    def sync(cls, context: Context, force: bool, arguments: Sequence[str] = ()) -> None:
+        cls.invoke(context, "refresh", [*(["--force"] if force else []), *arguments])
+
     @classmethod
     def createrepo(cls, context: Context) -> None:
-        run(["createrepo_c", context.packages],
-            sandbox=context.sandbox(binary="createrepo_c", mounts=[Mount(context.packages, context.packages)]))
+        run(["createrepo_c", context.repository],
+            sandbox=context.sandbox(binary="createrepo_c", mounts=[Mount(context.repository, context.repository)]))
 
         (context.pkgmngr / "etc/zypp/repos.d/mkosi-local.repo").write_text(
             textwrap.dedent(
                 """\
                 [mkosi]
                 name=mkosi
-                baseurl=file:///work/packages
+                baseurl=file:///repository
                 gpgcheck=0
                 autorefresh=0
                 keeppackages=0
@@ -170,4 +163,4 @@ class Zypper(PackageManager):
             )
         )
 
-        cls.invoke(context, "refresh", ["mkosi"])
+        cls.sync(context, force=True, arguments=["mkosi"])
