@@ -1,4 +1,4 @@
-# SPDX-License-Identifier: LGPL-2.1+
+# SPDX-License-Identifier: LGPL-2.1-or-later
 
 import contextlib
 import errno
@@ -149,7 +149,7 @@ def rmtree(*paths: Path, sandbox: SandboxProtocol = nosandbox) -> None:
     if not paths:
         return
 
-    if subvolumes := sorted({p for p in paths if is_subvolume(p, sandbox=sandbox)}):
+    if subvolumes := sorted({p for p in paths if p.exists() and is_subvolume(p, sandbox=sandbox)}):
         # Silence and ignore failures since when not running as root, this will fail with a permission error unless the
         # btrfs filesystem is mounted with user_subvol_rm_allowed.
         run(["btrfs", "subvolume", "delete", *subvolumes],
@@ -158,7 +158,7 @@ def rmtree(*paths: Path, sandbox: SandboxProtocol = nosandbox) -> None:
             stdout=subprocess.DEVNULL if not ARG_DEBUG.get() else None,
             stderr=subprocess.DEVNULL if not ARG_DEBUG.get() else None)
 
-    filtered = sorted({p for p in paths if p.exists()})
+    filtered = sorted({p for p in paths if p.exists() or p.is_symlink()})
     if filtered:
         run(["rm", "-rf", "--", *filtered],
             sandbox=sandbox(binary="rm", mounts=[Mount(p.parent, p.parent) for p in filtered]))
