@@ -1,4 +1,4 @@
-# SPDX-License-Identifier: LGPL-2.1+
+# SPDX-License-Identifier: LGPL-2.1-or-later
 
 import ast
 import contextlib
@@ -182,6 +182,10 @@ class StrEnum(enum.Enum):
     def values(cls) -> list[str]:
         return list(s.replace("_", "-") for s in map(str, cls.__members__))
 
+    @classmethod
+    def choices(cls) -> list[str]:
+        return [*cls.values(), ""]
+
 
 @contextlib.contextmanager
 def umask(mask: int) -> Iterator[None]:
@@ -305,12 +309,13 @@ def resource_path(mod: ModuleType) -> Iterator[Path]:
 
 def hash_file(path: Path) -> str:
     # TODO Replace with hashlib.file_digest after dropping support for Python 3.10.
-    bs = 16 * 1024**2
     h = hashlib.sha256()
+    b  = bytearray(16 * 1024**2)
+    mv = memoryview(b)
 
-    with path.open("rb") as sf:
-        while (buf := sf.read(bs)):
-            h.update(buf)
+    with path.open("rb", buffering=0) as f:
+        while n := f.readinto(mv):
+            h.update(mv[:n])
 
     return h.hexdigest()
 
