@@ -1,4 +1,4 @@
-# SPDX-License-Identifier: LGPL-2.1+
+# SPDX-License-Identifier: LGPL-2.1-or-later
 
 import argparse
 import itertools
@@ -23,7 +23,7 @@ from mkosi.config import (
     parse_config,
     parse_ini,
 )
-from mkosi.distributions import Distribution
+from mkosi.distributions import Distribution, detect_distribution
 from mkosi.util import chdir
 
 
@@ -97,16 +97,20 @@ def test_parse_config(tmp_path: Path) -> None:
     (d / "mkosi.conf").write_text(
         """\
         [Distribution]
-
-        @Distribution = ubuntu
-        Architecture  = arm64
+        Distribution=ubuntu
+        Architecture=arm64
+        Repositories=epel,epel-next
 
         [Content]
         Packages=abc
+        Environment=MY_KEY=MY_VALUE
 
         [Output]
-        @Format = cpio
-        ImageId = base
+        Format=cpio
+        ImageId=base
+
+        [Host]
+        Credentials=my.cred=my.value
         """
     )
 
@@ -120,44 +124,68 @@ def test_parse_config(tmp_path: Path) -> None:
     assert config.image_id == "base"
 
     with chdir(d):
-        _, [config] = parse_config(["--distribution", "fedora"])
+        _, [config] = parse_config(
+            [
+                "--distribution", "fedora",
+                "--environment", "MY_KEY=CLI_VALUE",
+                "--credential", "my.cred=cli.value",
+                "--repositories", "universe",
+            ]
+        )
 
-    # mkosi.conf sets a default distribution, so the CLI should take priority.
+    # Values from the CLI should take priority.
     assert config.distribution == Distribution.fedora
+    assert config.environment["MY_KEY"] == "CLI_VALUE"
+    assert config.credentials["my.cred"] == "cli.value"
+    assert config.repositories == ["epel", "epel-next", "universe"]
 
-    # Any architecture set on the CLI is overridden by the config file, and we should complain loudly about that.
-    with chdir(d), pytest.raises(SystemExit):
-        _, [config] = parse_config(["--architecture", "x86-64"])
+    with chdir(d):
+        _, [config] = parse_config(
+            [
+                "--distribution", "",
+                "--environment", "",
+                "--credential", "",
+                "--repositories", "",
+            ]
+        )
+
+    # Empty values on the CLIs resets non-collection based settings to their defaults and collection based settings to
+    # empty collections.
+    assert config.distribution == (detect_distribution()[0] or Distribution.custom)
+    assert "MY_KEY" not in config.environment
+    assert "my.cred" not in config.credentials
+    assert config.repositories == []
 
     (d / "mkosi.conf.d").mkdir()
     (d / "mkosi.conf.d/d1.conf").write_text(
         """\
         [Distribution]
-        Distribution = debian
-        @Architecture = x86-64
+        Distribution=debian
 
         [Content]
-        Packages = qed
-                   def
+        Packages=qed
+                 def
 
         [Output]
-        ImageId = 00-dropin
-        ImageVersion = 0
+        ImageId=00-dropin
+        ImageVersion=0
+        @Output=abc
         """
     )
 
     with chdir(d):
-        _, [config] = parse_config()
+        _, [config] = parse_config(["--package", "last"])
 
     # Setting a value explicitly in a dropin should override the default from mkosi.conf.
     assert config.distribution == Distribution.debian
-    # Setting a default in a dropin should be ignored since mkosi.conf sets the architecture explicitly.
-    assert config.architecture == Architecture.arm64
-    # Lists should be merged by appending the new values to the existing values.
-    assert config.packages == ["abc", "qed", "def"]
+    # Lists should be merged by appending the new values to the existing values. Any values from the CLI should be
+    # appended to the values from the configuration files.
+    assert config.packages == ["abc", "qed", "def", "last"]
     assert config.output_format == OutputFormat.cpio
     assert config.image_id == "00-dropin"
     assert config.image_version == "0"
+    # '@' specifier should be automatically dropped.
+    assert config.output == "abc"
 
     (d / "mkosi.version").write_text("1.2.3")
 
@@ -221,6 +249,42 @@ def test_parse_config(tmp_path: Path) -> None:
         assert config.bootable == ConfigFeature.enabled
         assert config.split_artifacts is False
 
+    (d / "mkosi.images").mkdir()
+
+    for n in ("one", "two"):
+        (d / "mkosi.images" / f"{n}.conf").write_text(
+            f"""\
+            [Distribution]
+            Repositories=append
+
+            [Content]
+            Packages={n}
+            """
+        )
+
+    with chdir(d):
+        _, [one, two, config] = parse_config(["--package", "qed", "--build-package", "def", "--repositories", "cli"])
+
+    # Universal settings should always come from the main image.
+    assert one.distribution == config.distribution
+    assert two.distribution == config.distribution
+    assert one.release == config.release
+    assert two.release == config.release
+
+    # Non-universal settings should not be passed to the subimages.
+    assert one.packages == ["one"]
+    assert two.packages == ["two"]
+    assert one.build_packages == []
+    assert two.build_packages == []
+
+    # But should apply to the main image of course.
+    assert config.packages == ["qed"]
+    assert config.build_packages == ["def"]
+
+    # list based settings should be appended to in subimages
+    assert one.repositories == ["append", "epel", "epel-next", "cli"]
+    assert two.repositories == ["append", "epel", "epel-next", "cli"]
+
 
 def test_parse_includes_once(tmp_path: Path) -> None:
     d = tmp_path
@@ -254,9 +318,9 @@ def test_parse_includes_once(tmp_path: Path) -> None:
         )
 
     with chdir(d):
-        _, [one, two] = parse_config([])
-        assert one.build_packages == ["abc", "def"]
-        assert two.build_packages == ["abc", "def"]
+        _, [one, two, config] = parse_config([])
+        assert one.build_packages == ["def"]
+        assert two.build_packages == ["def"]
 
 
 def test_profiles(tmp_path: Path) -> None:
@@ -296,21 +360,35 @@ def test_profiles(tmp_path: Path) -> None:
     assert config.distribution == Distribution.debian
     assert config.qemu_kvm == ConfigFeature.enabled
 
+    (d / "mkosi.conf").unlink()
+
+    with chdir(d):
+        _, [config] = parse_config(["--profile", "profile"])
+
+    assert config.profile == "profile"
+    # mkosi.conf.d/ should override the profile
+    assert config.distribution == Distribution.debian
+    assert config.qemu_kvm == ConfigFeature.enabled
+
 
 def test_override_default(tmp_path: Path) -> None:
     d = tmp_path
 
     (d / "mkosi.conf").write_text(
         """\
+        [Content]
+        Environment=MY_KEY=MY_VALUE
+
         [Host]
-        @ToolsTree=default
+        ToolsTree=default
         """
     )
 
     with chdir(d):
-        _, [config] = parse_config(["--tools-tree", ""])
+        _, [config] = parse_config(["--tools-tree", "", "--environment", ""])
 
     assert config.tools_tree is None
+    assert "MY_KEY" not in config.environment
 
 
 def test_local_config(tmp_path: Path) -> None:
@@ -320,6 +398,9 @@ def test_local_config(tmp_path: Path) -> None:
         """\
         [Distribution]
         Distribution=debian
+
+        [Content]
+        WithTests=yes
         """
     )
 
@@ -332,13 +413,24 @@ def test_local_config(tmp_path: Path) -> None:
         """\
         [Distribution]
         Distribution=fedora
+
+        [Content]
+        WithTests=no
         """
     )
 
     with chdir(d):
         _, [config] = parse_config()
 
+    # Local config should take precedence over non-local config.
+    assert config.distribution == Distribution.debian
+    assert config.with_tests
+
+    with chdir(d):
+        _, [config] = parse_config(["--distribution", "fedora", "-T"])
+
     assert config.distribution == Distribution.fedora
+    assert not config.with_tests
 
 
 def test_parse_load_verb(tmp_path: Path) -> None:
@@ -675,6 +767,25 @@ def test_match_build_sources(tmp_path: Path) -> None:
     assert config.output == "abc"
 
 
+def test_match_repositories(tmp_path: Path) -> None:
+    d = tmp_path
+
+    (d / "mkosi.conf").write_text(
+        """\
+        [Match]
+        Repositories=epel
+
+        [Content]
+        Output=qed
+        """
+    )
+
+    with chdir(d):
+        _, [config] = parse_config(["--repositories", "epel,epel-next"])
+
+    assert config.output == "qed"
+
+
 @pytest.mark.parametrize(
     "image1,image2", itertools.combinations_with_replacement(
         ["image_a", "image_b", "image_c"], 2
@@ -880,6 +991,29 @@ def test_package_manager_tree(tmp_path: Path, skel: Optional[Path], pkgmngr: Opt
         assert conf.package_manager_trees == pkgmngr_expected
 
 
+def test_paths_with_default_factory(tmp_path: Path) -> None:
+    """
+    If both paths= and default_factory= are defined, default_factory= should not
+    be used when at least one of the files/directories from paths= has been found.
+    """
+
+    with chdir(tmp_path):
+
+        Path("mkosi.skeleton.tar").touch()
+        _, [config] = parse_config()
+
+        assert config.package_manager_trees == [
+            ConfigTree(Path.cwd() / "mkosi.skeleton.tar", None),
+        ]
+
+        Path("mkosi.pkgmngr.tar").touch()
+        _, [config] = parse_config()
+
+        assert config.package_manager_trees == [
+            ConfigTree(Path.cwd() / "mkosi.pkgmngr.tar", None),
+        ]
+
+
 @pytest.mark.parametrize(
     "sections,args,warning_count",
     [
@@ -966,6 +1100,7 @@ def test_specifiers(tmp_path: Path) -> None:
                     ConfigRootDirectory=%D
                     ConfigRootConfdir=%C
                     ConfigRootPwd=%P
+                    Filesystem=%F
         """
     )
 
@@ -1008,6 +1143,7 @@ def test_specifiers(tmp_path: Path) -> None:
             "ConfigQedDirectory": os.fspath(d),
             "ConfigQedConfdir": os.fspath(d / "mkosi.conf.d/qed"),
             "ConfigQedPwd": os.fspath(d / "mkosi.conf.d/qed"),
+            "Filesystem": "ext4",
         }
 
         assert {k: v for k, v in config.environment.items() if k in expected} == expected
@@ -1064,9 +1200,13 @@ def test_environment(tmp_path: Path) -> None:
 
     (d / "mkosi.conf").write_text(
         """\
+        [Config]
+        PassEnvironment=PassThisEnv
+
         [Content]
         Environment=TestValue2=300
                     TestValue3=400
+                    PassThisEnv=abc
         EnvironmentFiles=other.env
         """
     )
@@ -1085,8 +1225,11 @@ def test_environment(tmp_path: Path) -> None:
         """
     )
 
+    (d / "mkosi.images").mkdir()
+    (d / "mkosi.images/sub.conf").touch()
+
     with chdir(d):
-        _, [config] = parse_config()
+        _, [sub, config] = parse_config()
 
         expected = {
             "TestValue1": "100", # from other.env
@@ -1099,3 +1242,6 @@ def test_environment(tmp_path: Path) -> None:
         assert {k: config.environment[k] for k in expected.keys()} == expected
 
         assert config.environment_files == [Path.cwd() / "mkosi.env", Path.cwd() / "other.env"]
+
+        assert sub.environment["PassThisEnv"] == "abc"
+        assert "TestValue2" not in sub.environment
