@@ -1,4 +1,4 @@
-# SPDX-License-Identifier: LGPL-2.1+
+# SPDX-License-Identifier: LGPL-2.1-or-later
 
 import os
 import textwrap
@@ -40,7 +40,6 @@ def test_args(path: Optional[Path]) -> None:
     dump = textwrap.dedent(
         f"""\
         {{
-            "Append": true,
             "AutoBump": false,
             "Cmdline": [
                 "foo",
@@ -62,7 +61,6 @@ def test_args(path: Optional[Path]) -> None:
     )
 
     args = Args(
-        append = True,
         auto_bump = False,
         cmdline = ["foo", "bar"],
         debug = False,
@@ -145,10 +143,6 @@ def test_config() -> None:
             "Image": "default",
             "ImageId": "myimage",
             "ImageVersion": "5",
-            "Images": [
-                "default",
-                "initrd"
-            ],
             "Include": [],
             "Incremental": false,
             "InitrdInclude": [
@@ -212,6 +206,9 @@ def test_config() -> None:
                 }
             ],
             "Packages": [],
+            "PassEnvironment": [
+                "abc"
+            ],
             "Passphrase": null,
             "PostInstallationScripts": [
                 "/bar/qux"
@@ -348,6 +345,9 @@ def test_config() -> None:
                 "Type": "file"
             },
             "VirtualMachineMonitor": "qemu",
+            "VolatilePackageDirectories": [
+                "def"
+            ],
             "VolatilePackages": [
                 "abc"
             ],
@@ -382,7 +382,7 @@ def test_config() -> None:
         compress_output=Compression.bz2,
         configure_scripts=[Path("/configure")],
         credentials= {"credkey": "credval"},
-        dependencies=("dep1",),
+        dependencies=["dep1"],
         distribution=Distribution.fedora,
         environment={"foo": "foo", "BAR": "BAR", "Qux": "Qux"},
         environment_files=[],
@@ -396,7 +396,6 @@ def test_config() -> None:
         image="default",
         image_id="myimage",
         image_version="5",
-        images=("default", "initrd"),
         include=[],
         incremental=False,
         initrd_include=[Path("/foo/bar"),],
@@ -432,6 +431,7 @@ def test_config() -> None:
         package_directories=[],
         package_manager_trees=[ConfigTree(Path("/foo/bar"), None)],
         packages=[],
+        pass_environment=["abc"],
         passphrase=None,
         postinst_scripts=[Path("/bar/qux")],
         postoutput_scripts=[Path("/foo/src")],
@@ -507,6 +507,7 @@ def test_config() -> None:
         verity_certificate=Path("/path/to/cert"),
         verity_key=None,
         verity_key_source=KeySource(type=KeySource.Type.file),
+        volatile_package_directories=[Path("def")],
         volatile_packages=["abc"],
         with_docs=True,
         with_network=False,
