@@ -1,15 +1,15 @@
-# SPDX-License-Identifier: LGPL-2.1+
+# SPDX-License-Identifier: LGPL-2.1-or-later
 
 from pathlib import Path
 
 import pytest
 
-from . import Image
+from . import Image, ImageConfig
 
 pytestmark = pytest.mark.integration
 
 
-def test_sysext(config: Image.Config) -> None:
+def test_sysext(config: ImageConfig) -> None:
     with Image(
         config,
         options=[
